-- soundeff.e: Sound Effects

constant SND_FILENAME = #00020000,
	 SND_ASYNC    = #00000001,
	 SND_LOOP     = #00000008

sequence sound_queue  -- sounds waiting to be played
sound_queue = {}

boolean playing_sound
playing_sound = FALSE

atom sound_can_stop, sound_must_stop
sound_can_stop = time()
sound_must_stop = sound_can_stop

atom backlog  -- number of seconds of sounds still to play
backlog = 0

global procedure task_sound_effect()
-- process a stream of sound frequencies and durations
    sequence next_sound
    atom now
    
    while TRUE do
	now = time()
	if length(sound_queue) = 0 then
	    -- no sounds waiting in the queue
	    if now >= sound_must_stop then
		-- stop current sound - it's been going long enough
		c_proc(PlaySound, {0, 0, 0}) -- quiet
		playing_sound = FALSE
		task_suspend(task_self())
	    else
		task_schedule(task_self(), {sound_must_stop - now +.01,
					    sound_must_stop - now +.02})
	    end if
	    task_yield()
	else
	    -- get next sound from queue
	    next_sound = sound_queue[1]
	    sound_queue = sound_queue[2..$]
	    
	    if now < next_sound[5] then
		-- it's not too late to start this sound
		c_proc(PlaySound, {next_sound[1], 0, 
		       or_all({SND_FILENAME, SND_ASYNC, next_sound[2]})})
		playing_sound = TRUE
		sound_can_stop = now + next_sound[3]
		sound_must_stop = now + next_sound[4]
		task_schedule(task_self(), {next_sound[3], next_sound[3]+0.1}) 
		task_yield()
	    end if
	end if
    end while
end procedure

procedure start_sound_effect(atom file_name, integer loop,
			     atom min_duration, atom max_duration, 
			     atom how_late)
-- start a new sound effect
    atom now
    
    if QUIET then
	return
    end if
    
    now = time()
    
    if not playing_sound or now >= sound_can_stop then
	-- start new sound right away
	c_proc(PlaySound, {file_name, 0, or_all({SND_FILENAME, SND_ASYNC, loop})})
	playing_sound = TRUE
	sound_can_stop = now + min_duration
	if max_duration = -1 then
	    sound_must_stop = now + 10000
	else    
	    -- schedule sound stop time
	    sound_must_stop = now + max_duration
	    task_schedule(t_sound_effect, {sound_must_stop - now, 
					   sound_must_stop - now + 0.1}) 
	end if
    else
	if time() + how_late < sound_can_stop then
	-- a sound is currently playing, queue this one
	    sound_queue = append(sound_queue, {file_name, loop,
				     min_duration, max_duration, now+how_late})
	    task_schedule(t_sound_effect, {sound_can_stop - now, 
					   sound_can_stop - now + 0.1}) 
     -- else 
	    -- ignore - it's too late for this sound
	end if
    end if
end procedure

global procedure errbeep()
-- error signal
end procedure

atom explode_file, red_alert_file, torpedo_file, docking_completed_file,
     phasor_file, low_energy_file, you_are_dead_file, galaxy_saved_file

explode_file = allocate_string("EXPLODE.WAV")
red_alert_file = allocate_string("REDALERT.WAV")
torpedo_file = allocate_string("TORPEDO.WAV")
docking_completed_file = allocate_string("DOCKING_COMPLETED.WAV")
phasor_file = allocate_string("PHASOR.WAV")
low_energy_file = allocate_string("LOW_ENERGY.WAV")
you_are_dead_file = allocate_string("YOU_ARE_DEAD.WAV")
galaxy_saved_file = allocate_string("Saved.WAV")

global procedure explosion_sound()
-- something is destroyed
    start_sound_effect(explode_file, 0, 1.5, -1, 1.0)
end procedure

global procedure red_alert()
-- something is destroyed
    start_sound_effect(red_alert_file, SND_LOOP, 3.0, 12.0, 4.0) 
end procedure

global procedure phasor_sound()
-- phasor hits enemy with n-unit blast
    start_sound_effect(phasor_file, 0, 0.0, -1, 1.0) 
end procedure

global procedure low_energy_sound()
-- phasor hits enemy with n-unit blast
    start_sound_effect(low_energy_file, 0, 3.5, -1, 20.0) 
end procedure

global procedure you_are_dead_sound()
-- phasor hits enemy with n-unit blast
    start_sound_effect(you_are_dead_file, 0, 6.0, -1, 25.0) 
end procedure

global procedure pod_sound(integer freq, atom duration)
-- pod in flight
end procedure

global procedure deflected_sound()
-- sound of a deflected torpedo
end procedure

global procedure torpedo_hit()
-- torpedo hits enemy
    phasor_sound() -- for now
end procedure

global procedure torpedo_sound()
-- torpedo launch
    start_sound_effect(torpedo_file, 0, 0.2, -1, 0.4) 
end procedure

global procedure Java_enter_sound()
-- Java enters quadrant
end procedure

global procedure Java_phasor_sound(integer freq, atom duration)
-- special Java phasor
    start_sound_effect(phasor_file, 0, 0.0, 0.1, 1.0) 
end procedure

global procedure docking_sound()
-- Euphoria successfully docks with a planet or base
    start_sound_effect(docking_completed_file, 0, 2.0, -1, 3.0) 
end procedure

global procedure victory_sound()
-- sound when you win the game 
    system("copy damaged.dat saved.wav", 2)
    start_sound_effect(galaxy_saved_file, SND_LOOP, 9.0, 10.0, 30.0) 
end procedure

